MODULE elements
   USE arrayutil, only:check_container

   implicit none

   integer, parameter :: NLEN_LAB = 8

   type :: element_type
      integer                 :: z         ! atomic number Z
      character(len=20)       :: name      ! name
      character(len=NLEN_LAB) :: lab       ! label
      real                    :: weight    ! atomic weight
      real                    :: c_radius  ! covalent radius
      real                    :: w_radius  ! van der Waals radius
      real, dimension(4)      :: ax        ! coefficients a per calcolo scattering factors (x-ray)
      real, dimension(4)      :: bx        ! coefficients b per calcolo scattering factors (x-ray)
      real                    :: cx        ! coefficients c per calcolo scattering factors (x-ray)
      real, dimension(3)      :: fMo       ! f',f'',mu/rho del Molibdeno
      real, dimension(3)      :: fCu       ! f',f'',mu/rho del Rame
      real, dimension(4)      :: ae        ! coefficients a per calcolo scattering factors (electron)
      real, dimension(4)      :: be        ! coefficients b per calcolo scattering factors (electron)
      real                    :: ce        ! coefficients c per calcolo scattering factors (electron)
      real                    :: fact      ! fattore di scattering dei neutroni
      real                    :: nw        ! number of elements
      integer                 :: radtype   ! tipo di radiazione usata
      integer                 :: charge    ! charge 
      integer                 :: ptab      ! pointer to table of elements
      real                    :: zeff
      real                    :: ifMottFormula
   end type element_type

! S subroutine read_chemical_elements()                 Load information about chemical elements 
! S integer function numelem(elem)                      Size di elem
! S function chemical_element_info(label)               Restituisce info su elemento chimico a partire dalla label
! F vdw_radius(zval) result(vdw)                        Restituisce il raggio di van der Waals da Z
! S ordina_elements(elem)                               Ordina le specie chimiche sulla base del fattore di scattering
! S print_elements(info,kpr)
! F function molecular_weight(pxen,pw) result(mw)       Compute Molecular Weigth from pointer to chemical table
! F content_volume(specv,nspecv)  result(vol)           Calcolo approssimato del volume del contenuto
! F z_from_specie(spec)  result(zval)                   Dalla specie al numero atomico 
! F z_from_pxen(pxen)  result(zval)                     Dalla posizione nel file xen al numero atomico
! F pxen_from_specie(spec)  result(pxen)                Dalla specie al numero atomico 
! S get_charge(lab,charge,z)                            Get charge and atomic number
! F pxen_from_charge(z,charge) result(pxen)             Restiruisce pxen con la carica più vicina
! F pxen_from_string(str)  result(pxen)                 Fornisce la specie carica più vicina
! F pxen_from_label(label) result(pxen)                 Cerca nelle stringa una specie chimica ed estrai il pxen
! F specie_from_pxen(z) result(spec)                    Dal puntatore al file xen alla specie
! F number_of_elem(zelem,zval)                          Numero di elementi con Z=zelem 
! F is_element(elem,zval) result(ptr)                   esiste la specie con z = zval
! S add_element(elem,zval,nw,add)                       Add new element to array only if it doesn't already esist
! S copy_elem(elem1,elem2)                              Copy elem2 in elem1
! F info_specie_from_pxen(zxen,nums,nspec,xspec)
! F oxidation_number(zval)  
! F electronegativity(zval) 
! S elem_from_atoms(elem,pxen)                          Estraggo elem dallo pxen di tutti gli atomi
! F equal_elements(elem1,elem2)  result(equal)          true se le due liste di elementi sono diverse
! F equal_element(elem1,elem2) result(equal)            true se i 2 elementi sono uguali
! S read_chemformula(sform,ier,elem,strfor)             Leggi una stringa contenente una formula chimica
! F contains_specie                                     check if zvet contains species
! S chemical_formula                                    generate chemical formula
! F elemental function average_volume(zval)             Get the average volume of element with atomic number zval
! S save_elem_bin(unitp,elem)                           Save elements on binary file
! S read_elem_bin(unitp,elem,ier)                       Read elements from binary file
! F s3s2_value(elem,wavetype)                           compute sum(ni*zi**3) / sqrt(sum(ni*zi**2))

!
!  Average volumes of elements (Act Cryst. (2002). B57, 489-493)
   real, dimension(100), parameter, private :: average_vol = (/                                      &
   5.08,10.00,                                                                                       &
!  H    He     
   22.6,36.,13.24,13.87,11.8,11.39,11.17,20.00,                                                      &
!  Li   Be  B     C     N    O     F     Ne
   26., 36.,39.6, 37.3, 29.5,25.2, 25.8, 30.00,                                                      &
!  Na   Mg  Al    Si    P    S     Cl    Ar
   36., 45.,42.,  27.3, 24.0,28.1, 31.9, 30.4, 29.4, 26.,26.9,39., 37.8,41.6,36.4,30.3, 32.7,40.00,  &
!  K    Ca  Sc    Ti    V    Cr    Mn    Fe    Co    Ni  Cu   Zn   Ga   Ge   As   Se    Br   Kr
   42., 47.,44.,  27.,  37., 38.,  38.,  37.3, 31.2, 35.,35., 51., 55., 52.8,48.0,46.7, 46.2,45.,    &
!  Rb   Sr  Y     Zr    Nb   Mo    Tc    Ru    Rh    Pd  Ag   Cd   In   Sn   Sb   Te    I    Xe
   46., 66.,                                                                                         &
!  Cs   Ba 
           58.,   54.,  57., 50.,  55.00,50.,  53.,  56.,45., 50., 42., 54., 49., 59.,  35.,         &
!          La     Ce    Pr   Nd    Pm    Sm    Eu    Gd  Tb   Dy   Ho   Er   Tm   Yb    Lu
                  40.,  43., 38.8, 42.7, 41.9, 34.3, 38.,43., 38.0,54., 52., 60., 50.00,55.00,60.00, &
!                 Hf    Ta   W     Re    Os    Ir    Pt  Au   Hg   Tl   Pb   Bi   Po    At    Rn
   70.00,60.00,                                                                                      &
!  Fr    Ra
           74.,   56.,  60., 58.,  45.,  70.00,17.,  70.00,70.00,70.00,70.00,70.00/)
!          Ac     Th    Pa   U     Np    Pu    Am    Cm    Bk    Cf    Es    Fm

   enum, bind(c)
   enumerator :: H_at=1, He_at, Li_at, Be_at, B_at, C_at, N_at, O_at, F_at, Ne_at,       &
                 Na_at, Mg_at, Al_at, Si_at, P_at, S_at, Cl_at, Ar_at, K_at, Ca_at,      &
                 Sc_at, Ti_at, V_at, Cr_at, Mn_at, Fe_at, Co_at, Ni_at, Cu_at, Zn_at,    &
                 Ga_at, Ge_at, As_at, Se_at, Br_at, Kr_at, Rb_at, Sr_at, Y_at, Zr_at,    &
                 Nb_at, Mo_at, Tc_at, Ru_at, Rh_at, Pd_at, Ag_at, Cd_at, In_at, Sn_at,   &
                 Sb_at, Te_at, I_at, Xe_at, Cs_at, Ba_at, La_at, Ce_at, Pr_at, Nd_at,    &
                 Pm_at, Sm_at, Eu_at, Gd_at, Tb_at, Dy_at, Ho_at, Er_at, Tm_at, Yb_at,   &
                 Lu_at, Hf_at, Ta_at, W_at, Re_at, Os_at, Ir_at, Pt_at, Au_at, Hg_at,    &
                 Tl_at, Pb_at, Bi_at, Po_at, At_at, Rn_at, Fr_at, Ra_at, Ac_at, Th_at,   &
                 Pa_at, U_at, Np_at, Pu_at, Am_at, Cm_at, Bk_at, Cf_at, Es_at, Fm_at,    &
                 Md_at, No_at, Lr_at, Rf_at, Db_at, Sg_at, Bh_at, Hs_at, Mt_at, Ds_at,   &
                 Rg_at, D_at
   endenum

   type(element_type), protected, dimension(:), allocatable :: eleminfo
   integer, dimension(0:111)                                :: pxen_from_z
   integer, protected                                       :: N_ELEMENTS = 0
   integer, parameter                                       :: MAXELEMENTS = 111

!corr   integer, parameter :: ORD_BY_RX=0, ORD_BY_NEUTRON=1, ORD_BY_ELECTRON=2
   integer, parameter :: RX_SOURCE=0, NEUTRON_SOURCE=1, ELECTRON_SOURCE=2

   interface pxen_from_specie
     module procedure pxen_from_specie_s, pxen_from_specie_v
   end interface

   interface z_from_specie
     module procedure z_from_specie_s, z_from_specie_v
   end interface

   interface chemical_formula
     module procedure chemical_formula_s, chemical_formula_sn, chemical_formula_el
   end interface

   interface contains_specie
     module procedure check_container, z_contains_specie_string
   end interface 

   interface add_element
      module procedure add_element_from_Z, add_element_from_el
   end interface


CONTAINS

   subroutine read_chemical_elements(filename)
!
!  Leggi elementi chimici da file esterno
!
   USE strutil
   USE fileutil
   USE iso_fortran_env
   character(len=*), intent(in) :: filename    ! nome del file
   type(file_handle)            :: fileh
   integer                      :: file_unit
   integer                      :: nline
   character(len=200)           :: line,line2
   integer                      :: ierror
   integer                      :: nlong1,nlong2
   integer                      :: nelem
   integer, dimension(20)       :: ivet
   real, dimension(20)          :: vet
   integer                      :: iv
   integer                      :: i
   integer                      :: ndimelem
!
   call fileh%fopen(filename)
   if (fileh%good()) then
       file_unit = fileh%handle()
!
!      Skip intro del file
       nline = 0
       do i=1,2
          nline = nline + 1
          read(file_unit,'(a)',iostat=ierror,err=10)line
          if (ierror < 0) go to 10
       enddo
!
!      Read commented block
       do 
          read(file_unit,'(a)',iostat=ierror,err=10)line
          if (.not.is_comment_line(line,'#')) exit
          nline = nline + 1
       enddo
       backspace(file_unit)
!
!      allocazione iniziale di elem
       ndimelem = 220
       call reallocate_elem(eleminfo,ndimelem,start=0)
!
       pxen_from_z(:) = 0
       nelem = 0
       loop_elements: do 
         do i=1,9
            nline = nline + 1
            read(file_unit,'(a)',iostat=ierror,err=10)line
            if (ierror < 0) exit loop_elements
            call s_filter(line)                   ! filtra le linea
            call cutst(line,nlong1,line2,nlong2)  ! taglia il simbolo chimico
            select case (i)
               case (1) ! lab Atomic   Weight    coval_radius    vanderWaals_radius
                 nelem = nelem + 1
                 if (nelem > ndimelem) then
                     ndimelem = ndimelem + 10
                     call reallocate_elem(eleminfo,ndimelem,start=0)
                 endif
                 line2(:1) = upper(line2(:1))    ! rendi maiuscolo il primo char. della label
                 eleminfo(nelem)%lab = line2(:nlong2)
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 4) go to 10
                 eleminfo(nelem)%z = ivet(1)
                 eleminfo(nelem)%weight = vet(2)
                 eleminfo(nelem)%c_radius = vet(3)
                 eleminfo(nelem)%w_radius = vet(4)
                 eleminfo(nelem)%ptab = nelem
                 if (pxen_from_z(ivet(1)) == 0) pxen_from_z(ivet(1)) = nelem

               case (2) !scattering factor (x-ray)
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 6) go to 10
                 eleminfo(nelem)%ax(:3) = vet(1:6:2)
                 eleminfo(nelem)%bx(:3) = vet(2:6:2)

               case (3) ! a(4),b(4),c x-ray
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 3) go to 10
                 eleminfo(nelem)%ax(4) = vet(1)
                 eleminfo(nelem)%bx(4) = vet(2)
                 eleminfo(nelem)%cx = vet(3)

               case(4)  ! f'(Mo),f"(Mo),mu/rho 
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 3) go to 10
                 eleminfo(nelem)%fMo(:) = vet(1:3)

               case(5)  ! f'(Cu),f"(Cu),mu/rho
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 3) go to 10
                 eleminfo(nelem)%fCu(:) = vet(1:3)

               case(6)  ! scattering factor (electrons)
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 6) go to 10
                 eleminfo(nelem)%ae(:3) = vet(1:6:2)
                 eleminfo(nelem)%be(:3) = vet(2:6:2)

               case(7)  ! a(4),b(4),c (electrons)
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 3) go to 10
                 eleminfo(nelem)%ae(4) = vet(1)
                 eleminfo(nelem)%be(4) = vet(2)
                 eleminfo(nelem)%ce = vet(3)

               case(8)  ! fact (neutrons)
                 call Getnum(line,vet,ivet,iv)
                 if (iv /= 1) go to 10
                 eleminfo(nelem)%fact = vet(1)

               case(9)  ! name
                 eleminfo(nelem)%name = trim(line)

            end select
         enddo
         call get_charge(eleminfo(nelem)%lab,eleminfo(nelem)%charge)
       enddo loop_elements
!
       call reallocate_elem(eleminfo,nelem,start=0)  ! ora rialloca elem al numero di specie lette
       eleminfo(:)%radtype = 0
       eleminfo(:)%nw = 0
       eleminfo(:)%zeff = 0
       eleminfo(:)%ifMottFormula = 0
       N_ELEMENTS = nelem
       call fileh%fclose()
       eleminfo(0)%z = 0
       eleminfo(0)%ptab = 0
       eleminfo(0)%lab = 'Q'
   else
       !if (.not.file_exist(filename)) then      
       !    write(0,*)'File '//trim(filename)//' does not exist'
       !endif
       write(ERROR_UNIT,'(a)')'Cannot open: '//trim(filename)
       write(ERROR_UNIT,'(a)')'Message: '//trim(fileh%err_msg())
   endif
   return
!
10 write(ERROR_UNIT,*)'Error on reading line ',nline,' in configuration file '//trim(filename)
   call fileh%fclose()
!
   end subroutine read_chemical_elements

!--------------------------------------------------------------------------------------------------

   subroutine reallocate_elem(vetr,n,savevet,start)
!
!  Rialloca ad n un vettore di tipo model
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(element_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                            :: n
   logical, intent(in), optional                  :: savevet
   integer, intent(in), optional                  :: start
   logical                                        :: savev
   integer, dimension(1)                          :: nv
   type(element_type), allocatable                :: vsav(:)
   integer                                        :: nsav
   integer                                        :: lb
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
   if (present(start)) then
       lb = start
   else
       lb = 1
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(lb:n))
   else
!
       nv = ubound(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv(1)-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv(1)
             case (0)
               return        ! n=nv non fare niente
           end select
           allocate(vsav(lb:n))
           vsav(:nsav) = vetr(:nsav)
           call move_alloc(vsav,vetr)
       else
           if (nv(1) /= n) then
               deallocate(vetr)
               allocate(vetr(lb:n))
           endif
       endif
   endif
!
   end subroutine reallocate_elem

!----------------------------------------------------------------------------------------

   integer function numelem(elem)
!
!  Size di elem
!
   type(element_type), allocatable, intent(in) :: elem(:)
   if (allocated(elem)) then
       numelem = size(elem)
   else
       numelem = 0
   endif
   end function numelem

!----------------------------------------------------------------------------------------

   function chemical_element(pxen,label,z) result(info)
!
!  Restituisce info su elemento chimico a partire 
!  dalla label (oppure dalla posizione nel file xen) e dal tipo di radiazione 
!  info%z = 0 se la label non corrisponde a nessuna specie chimica
!
   USE strutil
   integer, intent(in), optional          :: pxen   ! posizione della specie nel file xen
   character(len=*), intent(in), optional :: label  ! simbolo chimica
   integer, intent(in), optional          :: z      ! numero atomico
   type(element_type)                     :: info
   integer                                :: kfound
!
   if (present(pxen)) then
!
!      assegnata la posizione nel file xen
       if (pxen <= N_ELEMENTS) then
           kfound = pxen
       else
           kfound = 0
       endif
   elseif (present(label)) then
!
!      assegnata la label?
       if (present(label)) then
           kfound = pxen_from_specie(label)
       else
           kfound = 0
       endif
   else
       if (z > 0 .and. z <= ubound(pxen_from_z,dim=1)) then
           kfound = pxen_from_z(z)
       else
           kfound = 0
       endif
   endif
!
   if (kfound == 0) then   ! label non trovata
       info%z = 0
   else                    ! label trovata
       info = eleminfo(kfound)
   endif
!
   end function chemical_element

!--------------------------------------------------------------------

   real function vdw_radius(zval) result(vdw)
!
!  Restituisce il raggio di van der Waals
!
   integer, intent(in) :: zval   ! posizione della specie nel file xen
   integer             :: pxen
!
   pxen = pxen_from_z(zval)      
   if (pxen > 0) then
       vdw = eleminfo(pxen)%w_radius
   else
       vdw = 0
   endif
!
   end function vdw_radius

!--------------------------------------------------------------------

   subroutine ordina_elements(elem,radtype,sort,ord)
!
!  Ordina le specie chimiche sulla base del fattore di scattering
!
   USE nr
   USE realloc
   type(element_type), dimension(:), intent(inout) :: elem
   integer, intent(in)                             :: radtype
   integer, intent(in), optional                   :: sort
   integer, dimension(:), allocatable, intent(inout), optional :: ord
   integer, dimension(size(elem))                  :: iord
!
   if (size(elem) == 0) return
!
   select case (radtype)
      case (RX_SOURCE)          ! raggi X
         call indexx(elem(:)%z,iord)
      case (NEUTRON_SOURCE)     ! neutroni
         call indexx(abs(elem(:)%fact),iord)
      case (ELECTRON_SOURCE)    ! elettroni
         call indexx(elem(:)%zeff,iord)
   end select
   if (present(sort)) then
       if (sort < 0) iord = iord(size(elem):1:-1)
   endif
   elem(:) = elem(iord)
   if (present(ord)) then
       call reallocate(ord,size(elem),savevet=.false.)
       ord = iord
   endif
!
   end subroutine ordina_elements

!--------------------------------------------------------------------

   subroutine print_elements(elem,kpr,wave)
   type(element_type), dimension(:), intent(in) :: elem
   integer, intent(in)                          :: kpr
   integer, intent(in)                          :: wave 
   integer                                      :: i,n
   integer                                      :: nelem
   nelem = size(elem)
   write(kpr,'(/15x,a/)')'Unit cell contents and scattering factor constants'
   if (wave == 0) then
       write(kpr,'(12x,a)')'Atom      Symbol   Number in cell   Atomic number'
       do n=1,nelem
          write(kpr,'(10x,a,2x,a,i13,i16)')elem(n)%name(1:12),elem(n)%lab(1:4),nint(elem(n)%nw),elem(n)%z
       enddo
       write(kpr,'(/20x,a//,9x,a)')'f = sum (  a(i) * exp(-b(i)*s2)  )  i=1,4 + c ',  &
            'a(1)    b(1)    a(2)    b(2)    a(3)    b(3)    a(4)    b(4)     c'
       do n=1,nelem
          write(kpr,'(1x,a,9f8.3)')elem(n)%lab(1:4),(elem(n)%ax(i),elem(n)%bx(i),i=1,4),elem(n)%cx
       enddo
   else
       write(kpr,'(12x,a)')'Atom      Symbol   Number in cell   Atomic number     f'
       do n=1,nelem
          write(kpr,'(10x,a,2x,a,i13,i16,3x,f8.3)')elem(n)%name(1:12),elem(n)%lab(1:4),nint(elem(n)%nw),elem(n)%z,elem(n)%fact
       enddo
   endif
   end subroutine print_elements

!--------------------------------------------------------------------

   real function molecular_weight(pxen,pw) result(mw)
!
!  Compute Molecular Weigth from pointer to chemical table
!
   integer, dimension(:) :: pxen   ! pointer to chemical table
   real, dimension(:)    :: pw     ! contribution
   integer               :: i
   mw = 0
   do i=1,size(pxen)
      if (pxen(i) > 0) mw = mw + eleminfo(pxen(i))%weight*pw(i)
   enddo
   end function molecular_weight

!--------------------------------------------------------------------

   !real function content_volume(specv,nspecv)  result(vol)
   real function content_volume(ptab,nspecv)  result(vol)
!
!  Calcolo approssimato del volume del contenuto
!
   !character(len=*), dimension(:), intent(in) :: specv ! specie coinvol
   integer, dimension(:), intent(in) :: ptab
   real, dimension(:), intent(in)          :: nspecv   ! num. di atomi per specie
!corr   type(element_type), dimension(size(specv)) :: elm
   integer                                    :: i
!corr   integer                                    :: ptab
!
   vol = 0
   do i=1,size(ptab)
      !elm = chemical_element(label=specv(i))
      !ptab = pxen_from_specie(specv(i))
      !vol = vol + nspecv(i)*average_volume(elm(i)%z)
      vol = vol + nspecv(i)*average_volume(eleminfo(ptab(i))%z)
   enddo
!
   end function content_volume

!--------------------------------------------------------------------

    integer function z_from_specie_s(spec)  result(zval)
!
!   Dalla specie al numero atomico 
!
    USE strutil
    character(len=*), intent(in) :: spec
    integer                      :: i
!
    zval = 0
    do i=1,N_ELEMENTS
       if (s_eqi(spec,eleminfo(i)%lab)) then
           zval = eleminfo(i)%z
           exit
       endif
    enddo
!
    end function z_from_specie_s

!--------------------------------------------------------------------

    function z_from_specie_v(spec)  result(zval)
!
!   Dalla specie al numero atomico 
!
    USE strutil
    character(len=*), dimension(:), intent(in) :: spec
    integer, dimension(size(spec))             :: zval
    integer                                    :: i
!
    do i=1,size(spec)
       zval(i) = z_from_specie_s(spec(i))
    enddo
!
    end function z_from_specie_v

!------------------------------------------------------------------------------------

    elemental function z_from_pxen(pxen)  result(zval)
!
!   Dalla posizione nel file xen al numero atomico
!
    integer, intent(in) :: pxen
    integer             :: zval
!
    if(pxen > 0)then
       zval = eleminfo(pxen)%z
    else
       zval = 0
    endif
!
    end function z_from_pxen

!-------------------------------------------------------------------------------------------------

    integer function z_number(elem)
!
!   Extract z number 
!
    type(element_type), intent(in) :: elem
    z_number = eleminfo(elem%ptab)%z
    end function z_number

!-------------------------------------------------------------------------------------------------
   
    integer function pxen_from_specie_s(spec)  result(pxen)
!
!   Dalla specie al numero atomico 
!
    USE strutil
    character(len=*), intent(in) :: spec
    integer                      :: i
!
    pxen = 0
    do i=1,N_ELEMENTS
       if (s_eqi(spec,eleminfo(i)%lab)) then
           pxen = i
           exit
       endif
    enddo
!
    end function pxen_from_specie_s

!------------------------------------------------------------------------------------

    function pxen_from_specie_v(spec)  result(pxen)
    USE strutil
    character(len=*), dimension(:), intent(in) :: spec
    integer, dimension(size(spec))             :: pxen
    integer                                    :: j
    do j=1,size(spec)
       pxen(j) = pxen_from_specie_s(spec(j))
    enddo
    end function pxen_from_specie_v

!------------------------------------------------------------------------------------

    subroutine get_charge(lab,charge,z)
!
!   Get charge and atomic number
!
    USE strutil
    character(len=*), intent(in)   :: lab
    integer, intent(out)           :: charge
    integer, intent(out), optional :: z
    integer                        :: lenl
    integer                        :: i
    integer                        :: charge_sign, charge_value
    integer                        :: kal
    logical                        :: lsign, ldig
    character(len=1)               :: ch
    character(len_trim(lab))       :: lab1
    charge = 0
    charge_sign = 1
    charge_value = 0 
    if (present(z)) z = 0
    kal = 0
    ldig = .false.
    lsign = .false.
    lab1 = s_blank_delete(lab)
    lenl = len_trim(lab1)   
    i = 0
    do 
       i = i + 1
       if (i > lenl) exit
       ch = lab1(i:i)
       if (ch == ' ') cycle
       if (ch_is_alpha(ch)) then
           if (kal == 2) return         ! 3 caratteri consec.
           if (ldig .or. lsign) return  ! es. N+N, N2N
           kal = kal + 1
       elseif (ch_is_digit(ch)) then
           if (kal == 0) return  ! es. 2N
      !!     if (lsign) return     ! es. Na+2 Il segno va dopo la carica
           ldig = .true.
           call ch_to_digit(ch,charge_value)
       elseif (ch == '+' .or. ch == '-') then
           if (kal == 0) return  ! es. +Na
           lsign = .true.
           if (ch == '-') charge_sign = -1
       else
           return
       endif
    enddo
    charge = charge_value*charge_sign
    if (present(z)) then
        z = z_from_specie(lab1(1:kal))
    endif
    end subroutine get_charge

!------------------------------------------------------------------------------------

    integer function pxen_from_charge(z,charge) result(pxen)
!
!   Restiruisce pxen con la carica più vicina
!
    integer, intent(in) :: z, charge
    integer             :: i
    integer             :: diffmin, diff
    if (charge == 0) then
        pxen = pxen_from_z(z)
    else
        diffmin = 1000
        pxen = 0
        do i=1,N_ELEMENTS
           if (z == eleminfo(i)%z) then
               diff = abs(charge - eleminfo(i)%charge)
               if (diff == 0) then
                   pxen = i
                   exit
               elseif (diff < diffmin) then
                   diffmin = diff
                   pxen = i
               endif
           endif
        enddo
    endif
    end function pxen_from_charge

!------------------------------------------------------------------------------------

    integer function pxen_from_string(str)  result(pxen)
!
!   Dalla specie a pxen. Nel caso di specie cariche assenti fornisce la specie neutra più vicina
!
    USE strutil
    character(len=*), intent(in) :: str
    integer                      :: lens
    integer                      :: charge, zval
!    
    pxen = 0
    lens = len_trim(str)
    if (lens > 4) return   
    pxen = pxen_from_specie(str)
!
    if (pxen == 0) then
        call get_charge(str,charge,zval)
        pxen = pxen_from_charge(zval, charge)
    endif
!
    end function pxen_from_string

!------------------------------------------------------------------------------------

    integer function pxen_from_label(label) result(pxen)
!
!   Cerca nelle stringa una specie chimica ed estrai il pxen
!
    character(len=*), intent(in) :: label
    character(len_trim(label))   :: lab
    integer                      :: i
    integer                      :: lenl
!
    lab = adjustl(label)
    lenl = len_trim(lab)
    pxen = 0
    do i=min(lenl,4),1,-1
       pxen = pxen_from_specie(lab(1:i))
       if (pxen > 0) exit
    enddo
!
    end function pxen_from_label

!------------------------------------------------------------------------------------

    function specie_from_pxen(z) result(spec)
!
!   Dal puntatore al file xen alla specie
!
    character(len=NLEN_LAB) :: spec
    integer, intent(in)     :: z
!
    if (z > 0 .and. z <= N_ELEMENTS) then
        spec = eleminfo(z)%lab
    else
        spec = ' '
    endif
!
    end function specie_from_pxen

!-------------------------------------------------------------------------------------------------

    integer function number_of_elem(zelem,zval)  result(num)
!
!   Numero di elementi con Z=zelem 
!
    integer, intent(in)               :: zelem   ! number of atoms with Z=zelem
    integer, dimension(:), intent(in) :: zval    ! zval di tutti gli atomi
!
    num = count(zval(:) == zelem)
!
    end function number_of_elem

!-------------------------------------------------------------------------------------------------

    integer function is_element(elem,zval) result(ptr)
!
!   esiste la specie con z = zval. If exists return the pointer to elem array
!
    type(element_type), allocatable, intent(in) :: elem(:)
    integer, intent(in)                         :: zval
    integer                                     :: i
    ptr = 0
    do i=1,numelem(elem)
       if (elem(i)%z == zval) then
           ptr = i
           exit
       endif
    enddo
    end function is_element

!-------------------------------------------------------------------------------------------------

   subroutine copy_elem(elem1,elem2)
!
!  Copy elem2 in elem1
!
   type(element_type), dimension(:), allocatable, intent(inout) :: elem1
   type(element_type), dimension(:), allocatable, intent(in)    :: elem2
   integer                                                   :: nel2,nel1
!
   nel2 = numelem(elem2)
   nel1 = numelem(elem1)
   if (nel1 == nel2) then
       if (nel1 /= 0) elem1 = elem2
   else
       if (allocated(elem1)) deallocate(elem1)
       if (nel2 > 0) then
           allocate(elem1(nel2),source=elem2)
       endif
   endif
!
   end subroutine copy_elem

!-------------------------------------------------------------------------------------------------

    subroutine add_element_from_Z(elem,zval,nw,nsym,add)
!
!   Add new element to array only if it doesn't already esist
!
    type(element_type), allocatable, intent(inout) :: elem(:)   ! array of elements
    integer, intent(in)                            :: zval      ! z number to add
    integer, intent(in), optional                  :: nw        ! number for content
    logical, intent(out), optional                 :: add       ! true if new element was added
    integer, intent(in), optional                  :: nsym
    integer                                        :: nelem
!
    if (present(add)) add = .false.
    if (is_element(elem,zval) == 0) then   
        nelem = numelem(elem)
        call reallocate_elem(elem,nelem+1,savevet=.true.)
        elem(nelem+1) = chemical_element(z=zval)
        if (present(nw)) elem(nelem+1)%nw = nw
        if (present(nsym)) then
            if (nsym > 0) elem(nelem+1)%nw = nsym*elem(nelem+1)%nw
        endif
        if (present(add)) add = .true.
    endif
!
    end subroutine add_element_from_Z

!-------------------------------------------------------------------------------------------------

    subroutine add_element_from_el(elem,elemadd)
    type(element_type), dimension(:), allocatable, intent(inout) :: elem
    type(element_type), dimension(:), allocatable, intent(in)    :: elemadd
    integer                                                      :: neladd, nel, i, nel0
!
    neladd = numelem(elemadd)
    if (neladd == 0) return
    nel = numelem(elem)
    nel0 = nel
    call reallocate_elem(elem,nel+neladd)
    do i=1,neladd
       if (any(elem(:nel0)%ptab == elemadd(i)%ptab)) cycle
       nel = nel + 1
       elem(nel) = elemadd(i)
    enddo
    call reallocate_elem(elem,nel)
!
    end subroutine add_element_from_el

!-------------------------------------------------------------------------------------------------

    subroutine info_specie_from_pxen(zxen,nums,nspec,xspec)
    USE nr
    integer, dimension(:), intent(in)            :: zxen   ! puntatore al file xen per ogni atomo
    integer, intent(out)                         :: nums   ! numero di specie
    integer, dimension(:), intent(out), optional :: nspec  ! numero per ciscuna specie
    integer, dimension(:), intent(out), optional :: xspec  ! puntatore al file xen 
    integer, dimension(0:N_ELEMENTS)             :: zvet
    integer                                      :: zv
    integer, allocatable, dimension(:)           :: iord
    integer                                      :: i
!
    zvet(:) = 0
    do i=1,size(zxen)
       zv = zxen(i)
       zvet(zv) = zvet(zv) + 1
    enddo
!
    nums = 0
    do i=1,N_ELEMENTS
       if (zvet(i) /= 0) then
           nums = nums + 1
           if (present(nspec)) nspec(nums) = zvet(i)
           if (present(xspec)) xspec(nums) = i
       endif
    enddo
!
    if (nums > 0 .and. present(nspec) .and. present(xspec)) then
!
!       ordina per numero atomico
        allocate(iord(nums))
        call indexx(xspec(:nums),iord) 
        xspec(:nums) = xspec(iord)
        nspec(:nums) = nspec(iord)
    endif 
!
    end subroutine info_specie_from_pxen

!-------------------------------------------------------------------------------------------------

    subroutine elem_from_atoms(elem,pxen)
!
!   Estraggo elem dallo pxen di tutti gli atomi
!
    type(element_type), dimension(:), allocatable, intent(inout) :: elem
    integer, dimension(:), intent(in)                            :: pxen
    integer, dimension(0:N_ELEMENTS)                             :: zvet
    integer                                                      :: zv
    integer :: i
    integer :: nums
!
!   Calcola quanti per ogni specie
    zvet(:) = 0
    do i=1,size(pxen)
       zv = pxen(i)
       zvet(zv) = zvet(zv) + 1
    enddo
!
!   allocate elem
    !nums = 0
    !do i=1,N_ELEMENTS
    !   if (zvet(i) /= 0) then
    !       nums = nums + 1
    !   endif
    !enddo
    nums = count(zvet(1:) /= 0)
    call reallocate_elem(elem,nums,.false.)
!
!   fill elem
    nums = 0
    do i=1,N_ELEMENTS
       if (zvet(i) /= 0) then
           nums = nums + 1
           elem(nums) = eleminfo(i)
           elem(nums)%nw = zvet(i)
       endif
    enddo
!
    end subroutine elem_from_atoms

!-------------------------------------------------------------------------------------------------
 
    integer elemental function oxidation_number(zval)  result(ox)
    integer, intent(in) :: zval
    integer, dimension(104), parameter :: ox_number = (/              &
!          H                                                  He
            1,                                                 0,     &
!          Li Be                                B  C  N  O  F Ne
            1, 2,                               3, 4,-3,-2,-1, 0,     &
!          Na Mg                               Al Si  P  S Cl Ar
            1, 2,                               3, 4, 5, 4,-1, 0,     &
!           K Ca Sc Ti  V Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr
            1, 2, 3, 4, 5, 6, 4, 3, 3, 2, 2, 2, 3, 4, 3, 6,-1, 2,     &    
!          Rb Sr  Y Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te  I Xe
            1, 2, 3, 4, 5, 6, 7, 3, 3, 2, 1, 2, 3, 4, 3, 6,-1, 4,     &
!          Cs Ba 
            1, 2,                                                     &
!                La Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu
                  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,        &
!                   Hf Ta  W Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn
                     4, 5, 6, 4, 4, 3, 2, 3, 2, 3, 2, 3, 2,-1, 0,     &
!          Fr Ra
            1, 2,                                                     &
!                Ac Th Pa  U Np Pu Am Cm Bk Cf Es Fm Md No Lr Rf
                  3, 4, 5, 6, 5, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4 /)
    select case (zval)
       case (:0)
         ox = 0
       case (1:104)
         ox = ox_number(zval)
       case (105:)
         ox = 3
    end select

    end function oxidation_number

!-------------------------------------------------------------------------------------------------
 
    integer elemental function maximum_valence(zval)  result(mv)
!
!   Maximum valence of the elements
!
    integer, intent(in) :: zval
    integer, dimension(108), parameter :: max_valence = (/            &
!          H                                                  He
            1,                                                 0,     &
!          Li Be                                B  C  N  O  F Ne
            1, 2,                               3, 4, 5, 2, 1, 0,     &
!          Na Mg                               Al Si  P  S Cl Ar
            1, 2,                               3, 4, 5, 6, 7, 2,     &
!           K Ca Sc Ti  V Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr
            1, 2, 3, 4, 5, 6, 7, 6, 5, 4, 4, 2, 3, 4, 5, 6, 7, 2,     &    
!          Rb Sr  Y Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te  I Xe
            1, 2, 3, 4, 5, 6, 7, 8, 6, 6, 4, 2, 3, 4, 5, 6, 7, 8,     &
!          Cs Ba 
            1, 2,                                                     &
!                La Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu
                  3, 4, 4, 4, 3, 3, 3, 3, 4, 4, 3, 3, 3, 3, 3,        &
!                   Hf Ta  W Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn
                     4, 5, 6, 7, 8, 8, 6, 5, 4, 3, 4, 5, 6, 7, 6,     &
!          Fr Ra
            1, 2,                                                     &
!                Ac Th Pa  U Np Pu Am Cm Bk Cf Es Fm Md No Lr Rf Db Sg Bh Hs
                  3, 4, 5, 6, 7, 8, 7, 8, 4, 4, 4, 3, 3, 3, 3, 4, 5, 6, 7, 8/)
    select case (zval)
       case (:0)
         mv = 0
       case (1:108)
         mv = max_valence(zval)
       case (109:)
         mv = 3
    end select

    end function maximum_valence
!-------------------------------------------------------------------------------------------------

    real elemental function electronegativity(zval)  result(eneg)
    integer, intent(in) :: zval
    real, dimension(103), parameter :: electronegativity_value = (/    &
!          H                                                                                                     He
           2.20,                                                                                                 0.0,     &
!          Li    Be                                                                B     C     N     O     F     Ne
           0.98, 1.57,                                                             2.04, 2.55, 3.04, 3.44, 3.98, 0.0,     &
!          Na    Mg                                                                Al    Si    P     S     Cl    Ar
           0.93, 1.31,                                                             1.61, 1.90, 2.19, 2.58, 3.16, 0.0,     &
!          K     Ca    Sc    Ti    V     Cr    Mn    Fe    Co    Ni    Cu    Zn    Ga    Ge    As    Se    Br    Kr
           0.82, 1.31, 1.36, 1.54, 1.63, 1.66, 1.55, 1.83, 1.88, 1.91, 1.90, 1.65, 1.81, 2.01, 2.18, 2.55, 2.96, 3.00,  &
!          Rb    Sr    Y     Zr    Nb    Mo    Tc    Ru    Rh    Pd    Ag    Cd    In    Sn    Sb    Te    I     Xe
           0.82, 0.95, 1.22, 1.33, 1.6,  2.16, 1.9,  2.2,  2.28, 2.20, 1.93, 1.69, 1.78, 1.96, 2.05, 2.1,  2.66, 2.60,  &
!          Cs    Ba 
           0.96, 0.98,                                                &
!                La    Ce    Pr    Nd    Pm    Sm    Eu    Gd    Tb    Dy    Ho    Er    Tm    Yb    Lu
                 1.1,  1.12, 1.13, 1.14, 1.13, 1.17, 1.2,  1.2,  1.1,  1.22, 1.23, 1.24, 1.25, 1.1,  1.27,              &
!                Hf    Ta    W     Re    Os    Ir    Pt    Au    Hg    Tl    Pb    Bi    Po    At    Rn
                 1.3,   1.5, 2.36, 1.9,  2.2,  2.20, 2.28, 2.54, 2.00, 1.62, 1.87, 2.02, 2.0,  2.2,  2.2,                &  
!          Fr    Ra
           0.7,  0.92,                                                                                                  &
!                Ac    Th    Pa    U     Np    Pu    Am    Cm    Bk    Cf    Es    Fm    Md    No    Lr
                 1.1,  1.3,  1.5,  1.38, 1.36, 1.28, 1.13, 1.28, 1.3,  1.28, 1.3,  1.3,  1.3,  1.3,  1.3  /)
    select case (zval)
       case (:0)
         eneg = 0
       case (1:103)
         eneg = electronegativity_value(zval)
       case (104:)
         eneg = 1.3
    end select

    end function electronegativity

!-------------------------------------------------------------------------------------------------

    logical function equal_elements(elem1,elem2)  result(equal)
!
!   equal = true se le due liste di elementi sono diverse
!
    type(element_type), dimension(:), allocatable, intent(in) :: elem1,elem2
    integer                                                   :: nelem1,nelem2
    integer                                                   :: i,j
    logical                                                   :: eq_elem
!
    nelem1 = numelem(elem1)
    nelem2 = numelem(elem2)
    equal = .false.
    if (nelem1 == nelem2) then
        do i=1,nelem1
           do j=1,nelem2
              eq_elem = equal_element(elem1(i),elem2(j))
              if (eq_elem) exit
           enddo
           if (.not.eq_elem) exit
        enddo
        equal = eq_elem
    endif
!
    end function equal_elements

!-------------------------------------------------------------------------------------------------

    logical function equal_element(elem1,elem2) result(equal)
!
!   equal = true se i 2 elementi sono uguali
!
    USE strutil
    type(element_type), intent(in) :: elem1,elem2
    equal = s_eqi(elem1%lab,elem2%lab) .and. (elem1%nw == elem2%nw)
    end function equal_element

!-------------------------------------------------------------------------------------------------

    subroutine read_chemformula(sform,ier,elem,strfor)
!
!   Convert string in elem, elements are ordered according to file of elements
!
    character(len=*), intent(in)                       :: sform     ! chemical formula
    integer, intent(out)                               :: ier       ! error on reading formula on character ier
    type(element_type), dimension(:), allocatable, intent(out), optional :: elem  ! elementi nella formula
    character(len=*), intent(out), optional            :: strfor    ! formula in format with space (es. C 1 H 4)
    integer                                            :: nelem
    real, dimension(N_ELEMENTS)                        :: vsp
    integer                                            :: i
    type(element_type), dimension(:), allocatable :: elf
!
    vsp(:) = 0
    call read_string_formula(sform,vsp,ier)
    if (ier == 0 .and. (present(elem) .or. present(strfor))) then
        nelem = count(vsp /= 0) 
        if (nelem > 0) then
            call reallocate_elem(elf,nelem) 
            nelem = 0
            do i=1,N_ELEMENTS
               if (vsp(i) > 0) then
                   nelem = nelem + 1
                   elf(nelem) = eleminfo(i)
                   !elf(nelem)%nw = nint(vsp(i))
                   elf(nelem)%nw = vsp(i)    ! save real number 
                   !write(0,'(a,a,f0.3)')'SPEC='//trim(elf(nelem)%lab),' n=',elf(nelem)%nw
               endif
            enddo
        endif
        if (present(strfor)) call chemical_formula(elf,strfor,fform=1,ord=.false.,hide1=.false.)  ! show 1 after label
        if (present(elem) .and. nelem > 0) then
            call reallocate_elem(elem,nelem) 
            elem(:) = elf(:)
        endif
    endif
!
    end subroutine read_chemformula

!-------------------------------------------------------------------------------------------------
 
    logical function z_contains_specie_string(zval, specv)  result(okcont)
    integer, dimension(:), intent(in) :: zval
    character(len=*), dimension(:), intent(in) :: specv
    okcont = check_container(zval,z_from_specie(specv))
    end function z_contains_specie_string

!-------------------------------------------------------------------------------------------------

    subroutine chemical_formula_sn(specv,nspecv,fstring,fform,ord,hide1)
    USE strutil
    USE nr
    USE arrayutil
    character(len=*), dimension(:), intent(in) :: specv
    real, dimension(:), intent(in)             :: nspecv
    character(len=*), intent(out)              :: fstring
    integer, intent(in), optional              :: fform
    logical, intent(in), optional              :: ord
    logical, optional, intent(in)              :: hide1  ! 1 must be hidden if true
    integer                                    :: i,j
    integer                                    :: lens, numl
    character(len=10)                          :: strnum
    integer                                    :: kform
    real, dimension(size(specv))               :: elneg
    integer, dimension(size(specv))            :: iord, zval
    logical                                    :: kord
    integer                                    :: intval
    logical                                    :: hide11
!
    if (present(fform)) then
        kform = fform
    else
        kform = 1
    endif
    if (present(ord)) then
        kord = ord
    else
        kord = .true.
    endif
    if (present(hide1)) then
        hide11 = hide1
    else
        hide11 = .false.
    endif
    numl = size(specv)
    fstring = ' '
    if (numl > 0) then
        if (kord) then
!
!           Sort atom 
            zval(:) = z_from_specie(specv)
            if (contains_specie(zval,(/'H','C'/))) then
                do i=1,numl    ! for organic compound write C,H before other elements
                               ! sorted by electronegativity
                   select case (specv(i))
                     case ('H')
                       elneg(i) = 99
                     case ('C')
                       elneg(i) = 100
                     case default
                       elneg(i) = electronegativity(zval(i))
                   end select
                enddo
            else
                elneg(:) = zval(:)   
            endif       
            call indexx(elneg,iord)
            iord = iord(numl:1:-1)
        else
            iord = (/(i,i=1,numl)/)
        endif
        lens = len(fstring)
        do j=1,numl
           i = iord(j)
           if (nspecv(i) == 0) cycle
           intval = nint(nspecv(i))
           if (abs(intval - nspecv(i)) > 0.05 .or. intval == 0) then
               strnum = trim(r_to_s(nspecv(i),2))
           else
               if (intval == 1 .and. hide11) then
                   strnum = ' '
               else
                   strnum = trim(i_to_s(intval))
               endif
           endif
           if (kform == 1) then
               if (len_trim(fstring) + len_trim(specv(i)) + len_trim(strnum) + 2 <= lens) then
                   fstring = trim(fstring)//' '//trim(specv(i))//' '//trim(strnum)
               else
                   exit
               endif
           else
               if (len_trim(fstring) + len_trim(specv(i)) + len_trim(strnum) + 1 <= lens) then
                   fstring = trim(fstring)//' '//trim(specv(i))//trim(strnum)
               else
                   exit
               endif
           endif
        enddo
        fstring = adjustl(fstring)
    endif
!
    end subroutine chemical_formula_sn

!-------------------------------------------------------------------------------------------------

    
    subroutine chemical_formula_s(specv,fstring,fform,ord)    !!! FIXME - forse non serve
    USE strutil
    character(len=*), dimension(:), intent(in)    :: specv
    character(len=*), intent(out)                 :: fstring
    integer, intent(in), optional                 :: fform
    logical, intent(in), optional                 :: ord
    integer                                       :: numl
    type(element_type), dimension(:), allocatable :: elemf
    logical                                       :: kord
!
    numl = size(specv)
    if (numl > 0) then
        if (present(ord)) then
            kord = ord
        else
            kord = .true.
        endif
        call elem_from_atoms(elemf,pxen_from_specie(specv))
        if (present(fform)) then
            call chemical_formula_el(elemf,fstring,fform,kord)
        else
            call chemical_formula_el(elemf,fstring,fform,kord)
        endif
    else
        fstring = ' '
    endif
!
    end subroutine chemical_formula_s

!-------------------------------------------------------------------------------------------------

    subroutine chemical_formula_el(elem,string,fform,ord,hide1)
!
!   Convert elements in formula string
!
    USE strutil
    type(element_type), dimension(:), allocatable, intent(in) :: elem
    character(len=*), intent(inout)                           :: string
    integer, intent(in), optional                             :: fform
    logical, intent(in), optional                             :: ord
    logical, intent(in), optional                             :: hide1
    logical                                                   :: kord,hide11
!
    if (numelem(elem) > 0) then
        if (present(ord)) then
            kord = ord
        else
            kord = .true.
        endif
        if (present(hide1)) then
            hide11 = hide1
        else
            hide11 = .false.
        endif
        if (present(fform)) then
            call chemical_formula_sn(elem%lab,elem%nw,string,fform,kord,hide11)
        else
            call chemical_formula_sn(elem%lab,elem%nw,string,1,kord,hide11)
        endif
    else
        string = ' '
    endif
!
    end subroutine chemical_formula_el

!-------------------------------------------------------------------------------------------------

    recursive subroutine read_string_formula(sform,vsp,ier)
!
!   recursive function to convert a formula string in vsp vector
!
    USE strutil
    character(len=*), intent(in) :: sform
    real, dimension(:), intent(inout) :: vsp
    integer, intent(out)         :: ier
    integer                      :: lens
    character(len=1)             :: ch
    integer                      :: i
    integer                      :: cini,cfin
    integer                      :: kal, kdig
    integer, parameter           :: MAXDIGIT = 10
    integer, parameter           :: MAXLAB = 10
    character(len=MAXDIGIT)      :: digit
    character(len=MAXLAB)        :: label
    integer                      :: len
    integer                      :: ieri
    real                         :: num
    integer                      :: lastch
    logical                      :: readf, lsign
    integer, parameter           :: CALPHA=1,CDIGIT=2,CSIGN=3,CBLANK=4,CBRACKET=5,CPOINT=6
    character(len=len_trim(sform)) :: sformb
    integer                        :: pxen
    logical                        :: kpr = .false.
    real                           :: numb
    real, dimension(size(vsp))     :: vsp1
    integer                        :: bpos
!
    ier = 0
    lens = len_trim(sform)
    if (lens == 0) return
    !vsp(:) = 0
!   
    kal = 0
    kdig = 0
    cini = 0
    cfin = 0
    lsign = .false.
    lastch = 0
    label = ' '
    i = 0
    do 
       i = i + 1
       if (i > lens) exit
       ch = sform(i:i)
       if (ch_is_alpha(ch)) then
           if (cini == 0) then
               cini = i
           else
               readf = .false.
               if (kal == 2) then
                   readf = .true.
               elseif (kal == 1) then
!
                   if (lastch == CALPHA) then
!                      2 caratteri consecutivi. Valuta! 
!                      Es. SB e sB diventano S e B. Sb e sb diventano Sb
                       if (ch_is_upper(ch)) then
                           readf = .true.
                           if (i == lens) then   ! ultimo carattere
                               if (pxen_from_string(ch) == 0) readf = .false.  ! es. CL
                           else
                               if (.not.ch_is_alpha(sform(i+1:i+1))) then           ! il carattere successivo non e' una lettera
                                   if (pxen_from_string(ch) == 0) readf = .false.  ! es. 'CL C' ma non 'CLa'
                               endif
                           endif
                      !     if (pxen_from_string(ch) /= 0) then  ! controlla anche che ch sia ultimo o abbia spazio dopo
                      !     readf = .true.
                      !     endif
                       endif
                   else
                       readf = .true.
                   endif
               endif
               if (readf) then
                   if (kdig > 0 .and. digit(:kdig) /= '.') then  
                       call s_to_r(digit(:kdig),num,ieri,len)
                          !write(0,*)'ieri=',ieri,trim(digit(:kdig)),num,len
                   else
                       num = 1
                   endif
                   pxen = pxen_from_string(label(:cfin))
                   if (kpr) write(0,'(a,i0,a,f0.1)')'pxen di '//label(:cfin)//'=',pxen,' num=',num
                   if (pxen > 0) then
                       vsp(pxen) = vsp(pxen) + num
                       cini = i
                       cfin = 0
                       lsign = .false.
                       kal = 0
                       kdig = 0
                       label = ' '
                   else
                       ier = cini
                       exit
                   endif
               endif
           endif
           cfin = cfin + 1
           if (cfin - cini + 1 > MAXLAB) then
               cfin = MAXLAB
               ier = i
               exit
           endif
           kal = kal + 1
           label(kal:kal) = ch
           lastch = CALPHA
       elseif (ch_is_digit(ch) .or. ch == '.') then
           if (cini == 0 .or. (kdig > 0 .and. lastch == CBLANK) .or. kdig == MAXDIGIT .or.    &
               (kdig > 0 .and. lastch == CPOINT .and. ch == '.')) then
!
!              la prima cifra è un numero (es. 1N) o secondo numero dopo spazio (Ca2+ 2 3...)
!              o piu' punti consecutivi (Ca..)
               ier = i
               exit
           endif
           kdig = kdig + 1
           digit(kdig:kdig) = ch
           if (ch == '.') then
               lastch = CPOINT
           else
               lastch = CDIGIT
           endif
       elseif (ch == '+' .or. ch == '-') then
           if (cini == 0 .or. lsign) then
               ier = i
               exit
           else
               cfin = cfin + kdig + 1
               if (cfin - cini + 1 > MAXLAB) then
                   ier = i
                   exit
               endif
               label = trim(label)//digit(:kdig)//ch
               digit(:kdig) = ' '
               kdig = 0
           endif
           lsign = .true.
           lastch = CSIGN
       elseif (ch_is_space(ch)) then
           lastch = CBLANK
       elseif (ch == '(' .or. ch == '[' .or. ch == '{') then 
           call get_string_in_brackets(sform(i:),sformb,bpos,ier)
           if (ier ==  0) then
               vsp1(:) = 0
               call read_string_formula(sformb,vsp1,ier)
               if (ier == 0) then
                 !  i = i + len_trim(sformb) + 1  ! bracket position
                   i = i + bpos - 1  ! bracket position
                   call get_next_number(sform,i,numb,ieri)  ! get number after bracket
                   if (ieri == 0) then    ! numero trovato
                       vsp(:) = vsp(:) + vsp1(:)*numb
                   else                   ! numero non trovato
                       vsp(:) = vsp(:) + vsp1(:)
                   endif
               else  ! errore nella parentesi
                   ier = i+ier
                   exit
               endif
           else   ! parentesi non chiusa
               ier = i
               exit
           endif
       else
           ier = i
           exit
       endif
    enddo
!
    if (cfin > 0) then
        if (kdig > 0 .and. digit(:kdig) /= '.') then  
            call s_to_r(digit(:kdig),num,ieri,len)
                          !write(0,*)'1 ieri=',ieri,trim(digit(:kdig)),num,cini
        else
            num = 1
        endif
        pxen = pxen_from_string(label(:cfin))
        if (pxen > 0) then
            vsp(pxen) = vsp(pxen) + num
            if (kpr) write(0,'(a,i0,a,f0.1)')'pxen di '//label(:cfin)//'=',pxen_from_string(label(:cfin)),' num=',num
        else
            ier = cini
        endif
    endif
    if (ier > 0 .and. kpr) then
        write(0,*)'error on ch.'//sform(ier:ier),ier
    endif
!
    end subroutine read_string_formula

!-------------------------------------------------------------------------------------------------

    real elemental function average_volume(zval)  result(vol)
!
!   Get the average volume of element with atomic number zval
!
    integer, intent(in) :: zval
    select case (zval)
       case (:0)
          vol = 0
       case (1:100)
          vol = average_vol(zval)
       case (101:)
          vol = 70
    endselect
    end function average_volume

!-------------------------------------------------------------------------------------------------

    subroutine save_elem_bin(unitp,elem)
!
!   Save elements on binary file
!
    integer, intent(in)                                       :: unitp
    type(element_type), dimension(:), allocatable, intent(in) :: elem
    integer                                                   :: nelem
    nelem = numelem(elem)
    write(unitp)nelem
    if (nelem > 0) write(unitp) elem(:)
    end subroutine save_elem_bin

!-------------------------------------------------------------------------------------------------

    subroutine read_elem_bin(unitp,elem,err)
!
!   Read elements from binary file
!
    USE errormod
    integer, intent(in)                                        :: unitp
    type(element_type), dimension(:), allocatable, intent(out) :: elem
    type(error_type), intent(out)                              :: err
    integer                                                    :: ier
    integer                                                    :: nelem
!
    ier = 0
    read(unitp,iostat=ier) nelem
    if (nelem > 0 .and. ier == 0) then
        call reallocate_elem(elem,nelem)
        read(unitp,iostat=ier), elem(:)
    endif
    if (ier /= 0) then
        call err%set('Error on reading cell content')
    endif
!
    end subroutine read_elem_bin

!-------------------------------------------------------------------------------------------------

    real function s3s2_value(elem,wavetype) result(s3s2)
    type(element_type), dimension(:), allocatable, intent(in) :: elem
    integer, intent(in)                                       :: wavetype
    integer :: nelem,i 
    real :: ss,s2,s3
!
    nelem = numelem(elem)
    if (nelem > 0) then
        s2 = 0.0
        s3 = 0.0
        if (wavetype == 0) then    ! X-Ray
            do i=1,nelem
               ss = elem(i)%nw*elem(i)%z*elem(i)%z
               s2 = s2 + ss
               s3 = s3 + ss*elem(i)%z
            enddo
        else                       ! Neutron
            do i=1,nelem
               ss = elem(i)%nw*elem(i)%fact*elem(i)%fact
               s2 = s2 + ss
               s3 = s3 + ss*elem(i)%fact
            enddo
        endif
        s3s2 = s3/sqrt(s2**3)
    else
        s3s2 = 0.0
    endif
!
    end function s3s2_value

END MODULE elements
