MODULE reflection_type_util
USE progtype, only:reflection_type

implicit none

type gref_type
  integer, dimension(:), allocatable :: vref  ! riflessi del gruppo
  integer                            :: code  ! num. di riflessi
  real                               :: Io    ! int. osservata
  real                               :: Ic    ! int. calcolata
end type

!S fillref(refl)                                          Scarica refl nelle variabili expo
!S sftref(refl)                                           Calcola Fc usando refl
!S defovlexr(refl,sogh,indov)                             Calcola overlapping
!S genera_gruppiref(refl,gref,sogi,selcodei)              Genera gruppi di riflessi in overlapping
!S print_gruppi(gref)                                     Stampa i gruppi
!S update_int_gruppi(refl,gref,codei)                     Aggiorna intensita' dei gruppi
!F number_of_group(nref,gref)  result(num)                Trova a quale gruppo appartine il riflesso nref
!S print_fcalc(refl)                                      Stampa info sui riflessi
!S write_reflections(jfile,fname,refl,code)               Stampa i riflessi
!S statvet(svety,svetx,nbin)                              Esegue statistica sul vettore svety ordinato  
!S set_reflection_table(table,vet,ier)                    Usata per generare una tabella di informazioni sui riflessi
!F LPcorrection(tthd)                                     Correzione Lorentz-polarizzazione
!S sort_reflections(ref,key)                              sort reflections
!S save_refl_bin(unitbin,refl)                            write reflections on binary file
!S read_refl_bin(unitbin,refl,ier)                        read reflections from binary file
!S select_ref(refl,code) --FIXME--                        Seleziona i reflessi in base ad un criterio definito da code  

integer, parameter :: ORD_BY_2THETA=1, ORD_BY_INT=2

 CONTAINS

   subroutine fillref(refl)
!
!  Scarica refl nelle variabili expo
!
!corr   USE progtype
   USE General  , only : nrefl
   USE Riflessi, only : khl,mult,rhov,fovet,ipub,iphaz
   USE realloc
!
   implicit none
   type(reflection_type), dimension(:), allocatable, intent(inout) :: refl
   integer                                                         :: i
!
   call reallocate(refl,nrefl,.false.)
   do i=1,nrefl
      refl(i)%hkl = khl(1:3,i)            ! indici h,k,l
      refl(i)%m   = mult(i)               ! molteplicità
      refl(i)%slaq = rhov(i)              ! (sen(theta)/lambda)**2
      refl(i)%fo  = fovet(i)              ! fattori di struttura osservati
      refl(i)%ph = iphaz(i)                                 ! fasi calcolate
      refl(i)%phv = ipub(i)                                 ! fasi vere
   enddo
!
   end subroutine fillref

!---------------------------------------------------------------------------------------

   integer function numrefl(refl) result(nref)
   type(reflection_type), dimension(:), allocatable, intent(in) :: refl
   if (allocated(refl)) then
       nref = size(refl)
   else
       nref = 0
   endif
   end function numrefl

!---------------------------------------------------------------------------------------

   subroutine sftref(refl,atom)
!
!  Calcola Fc usando refl
!
   USE realloc
   USE atomi
   USE factor
   USE atom_type_util
   type(reflection_type), allocatable, dimension(:), intent(inout) :: refl
   type(atom_type), dimension(:), intent(in), optional             :: atom
   type(atom_type), dimension(:), allocatable                      :: atmb
!
!  alloca e riempi reflessi
   call fillref(refl)
!
!  alloca e riempi atomi
   if (present(atom)) then  
       call reallocate(atmb,size(atom))
       atmb(:) = atom(:)
   else
       call xyz_to_atom(atmb,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:))
   endif
!
!  rimuovi eventuali ghosts
   call remove_atoms_from_list(atmb,atmb%nz,0)
   if (numatomtype(atmb) == 0) return
!
!  calcola fattori di struttura
   call fcalcan(refl,atmb)
!
   end subroutine sftref

!---------------------------------------------------------------------------------------

   subroutine fcalcang(refl,atom,anis)  
!
!  Calcola fattori di struttura in presenza di ghost atoms
!
   USE atom_type_util
   USE factor
   type(reflection_type), dimension(:), intent(inout)     :: refl
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(atom_type), dimension(:), allocatable             :: atom1
   logical, intent(in), optional                          :: anis
   logical                                                :: banis
!
   banis = .false.
   if (present(anis)) banis = anis
   if (any(atom(:)%nz == 0)) then    ! ci sono ghost atoms?
       call copia_atoms(atom1,atom)
!
!      rimuovi ghosts
       call remove_atoms_from_list(atom1,atom1%nz,0)
!
       if (numatomtype(atom1) > 0) then
           if (banis) then
               call fcalcan_anis(refl,atom1)
           else
               call fcalcan(refl,atom1)
           endif
       else                     ! caso limite: ho solo ghost atoms
           refl(:)%fc = 0.
           refl(:)%ph = 0
       endif
   else
       if (banis) then
           call fcalcan_anis(refl,atom)
       else
           call fcalcan(refl,atom)
       endif
   endif
!
   end subroutine fcalcang

!---------------------------------------------------------------------------------------

   subroutine defovlexr(refl,sogh,indov)
!
!  Calcola overlapping
!
   type(reflection_type), dimension(:), intent(in) :: refl
   real, intent(in)                                :: sogh
   integer, intent(out), dimension(:)              :: indov
   integer                                         :: js,jg
   integer                                         :: ifl
   integer                                         :: nr
!
   nr = size(indov)
   indov = 1
!
   DO JS=1,nr-1
      If(indov(JS).EQ.0) cycle
      indov(JS) = 1
      DO JG=JS+1,nr
         IFL = 0
         IF(ABS(refl(jg)%tthd-refl(jg-1)%tthd) <= sogh*refl(jg)%fwhm) THEN
            indov(JS) = indov(JS)+1
            indov(JG) = 0
            IFL = 1
         END IF
         IF(IFL == 0) exit
      enddo
   enddo
!
   end subroutine defovlexr

!---------------------------------------------------------------------------------------

   subroutine genera_gruppiref(refl,gref,sogi,selcodei)
!
!  Genera gruppi di riflessi in overlapping
!
   type(reflection_type),        dimension(:), intent(in)    :: refl     ! riflessi
   type(gref_type), allocatable, dimension(:), intent(inout) :: gref     ! gruppi
   real, intent(in), optional                                :: sogi     ! soglia di overlap
   integer, intent(in), optional                             :: selcodei ! seleziona i gruppi con code >= selcode
   integer, dimension(size(refl))                            :: over
   real                                                      :: sog
   integer                                                   :: selcode
   integer                                                   :: i
   integer                                                   :: nrrf
   integer                                                   :: ngref
   integer                                                   :: n
!
   nrrf = size(refl)
!
!  Calcola sovrapposizione tra i riflessi
   if (present(sogi)) then
       sog = sogi
   else
       sog = 0.6
   endif
   call defovlexr(refl,sog,over)
!
!  Seleziona i gruppi con code >= selcode
   if (present(selcodei)) then
       selcode = selcodei
   else
       selcode = 1
   endif
!
!  alloca gref
   ngref = 0
   do i=1,nrrf
      if (over(i) >= selcode) then
          ngref = ngref + 1
      endif
   enddo
   if (allocated(gref)) deallocate(gref)
   allocate(gref(ngref))
!
!  riempi gref
   ngref = 0
   n = 0
   do 
     n = n + 1
     if (n > nrrf) exit
     if (over(n) >= selcode) then
         ngref = ngref + 1
         gref(ngref)%code = over(n)
         allocate(gref(ngref)%vref(over(n)))
         gref(ngref)%vref(1) = n
         do i=2,gref(ngref)%code
            n = n + 1
            gref(ngref)%vref(i) = n
         enddo
     endif
   enddo
   call update_int_gruppi(refl,gref,3)
!
   end subroutine genera_gruppiref

!---------------------------------------------------------------------------------------

   subroutine print_gruppi(gref,kpri)
   type(gref_type), dimension(:), intent(inout) :: gref  ! gruppi
   integer, intent(in), optional                :: kpri
   integer                                      :: i
   character(len=10)                            :: strnum
   character(len=100)                           :: strf
   integer                                      :: kpr
!
   if (present(kpri)) then
       kpr = kpri
   else
       kpr = 6
   endif
   write(kpr,'(a)')'   n.        Io         Ic         code           ref'
   do i=1,size(gref)
      write(strnum,'(i0,"i5")')gref(i)%code
      strf = '(i5,2f15.2,i4,'//trim(strnum)//')'
      write(kpr,strf)i,gref(i)%Io,gref(i)%Ic,gref(i)%code,gref(i)%vref(:)
   enddo
!
   end subroutine print_gruppi

!---------------------------------------------------------------------------------------

   subroutine update_int_gruppi(refl,gref,codei)
   type(reflection_type), dimension(:), intent(in) :: refl  ! riflessi
   type(gref_type), dimension(:), intent(inout)    :: gref
   integer, intent(in), optional                   :: codei
   integer                                         :: code
   integer                                         :: i
!
   if (present(codei)) then
       code = codei
   else
       code = 3
   endif
   select case(code)
      case (1)                
        do i=1,size(gref)
           gref(i)%Io = sum(refl(gref(i)%vref)%m*refl(gref(i)%vref)%lp*refl(gref(i)%vref)%fo**2)
        enddo

      case (2)
        do i=1,size(gref)
           gref(i)%Ic = sum(refl(gref(i)%vref)%m*refl(gref(i)%vref)%lp*refl(gref(i)%vref)%fc**2)
        enddo

      case (3)
        do i=1,size(gref)
           gref(i)%Io = sum(refl(gref(i)%vref)%m*refl(gref(i)%vref)%lp*refl(gref(i)%vref)%fo**2)
           gref(i)%Ic = sum(refl(gref(i)%vref)%m*refl(gref(i)%vref)%lp*refl(gref(i)%vref)%fc**2)
        enddo
   end select
!
   end subroutine update_int_gruppi

!---------------------------------------------------------------------------------------

   integer function number_of_group(nref,gref)  result(num)
!
!  Trova a quale gruppo appartine il riflesso nref
!
   integer, intent(in)                       :: nref
   type(gref_type), dimension(:), intent(in) :: gref
   integer                                   :: i,j
!
   do i=1,size(gref)
      do j=1,gref(i)%code
         if (gref(i)%vref(j) == nref) then 
             num = i
             exit
         endif
      enddo
   enddo
!
   end function number_of_group

!---------------------------------------------------------------------------------------

   subroutine print_fcalc(refl)
   type(reflection_type), dimension(:), intent(in) :: refl
   integer                                         :: i
   real                                            :: scalf
   real, dimension(size(refl))                     :: vetf
!   
   scalf = sum(refl%fv)/sum(refl%fo)
   write(6,*)'  n       hkl       2-theta      fo     scal*fo     fv abs(fv-fo) phv'
   do i=1,size(refl)
      write(6,'(i5,3i5,5f10.3,i5)')i,refl(i)%hkl,refl(i)%tthd,refl(i)%fo,         &
          scalf*refl(i)%fo,refl(i)%fv,abs(refl(i)%fv-scalf*refl(i)%fo),refl(i)%phv
   enddo
   vetf = abs(refl%fv - scalf*refl%fo) / sum(refl%fv)
   write(6,*)'RF=',sum(abs(refl%fv-scalf*refl%fo))/sum(refl%fv),sum(vetf)
   call statvet(vetf,refl%tthd,5)
!   
   end subroutine print_fcalc

!----------------------------------------------------------------------------------------------------

   subroutine write_reflections(jfile,fname,refl,code,nref)
!
!  Stampa i riflessi
!        code = 1   ! h  k  l
!        code = 2   ! h  k  l   Fo^2  sigma
!        code = 3   ! h  k  l   Fc^2  sigma
!        code = 4   ! h  k  l   Fo    Fc   phase   (file fc)
!        code = 5   ! h  k  l   d     mult  lp   Fc^2    I      I(%)
   USE fileutil
!corr   USE counts
           !USE, intrinsic :: ieee_arithmetic
   type(file_handle), intent(in), optional         :: jfile
   character(len=*), intent(in), optional          :: fname
   type(file_handle)                               :: fileh
   type(reflection_type), dimension(:), intent(in) :: refl
   integer, intent(in)                             :: code
   integer, optional, intent(in)                   :: nref
   real                                            :: std
   integer                                         :: nrefw
   integer                                         :: i
   real, dimension(:), allocatable                 :: inte
   type(reflection_type), dimension(:), allocatable :: reflc
   real                                            :: maxinte
   !type(gref_type), dimension(:), allocatable :: gref
!
   if (present(fname)) then
       call fileh%fopen(fname,'w')
   else
       fileh = jfile
   endif
!
   nrefw = size(refl)
   if (present(nref)) nrefw = min(nref,nrefw)
   std = 0.01
   select case (code)
      case (1)      ! h k l
        write(fileh%handle(),'(i5,a1,3i4)')(i,')',refl(i)%hkl,i=1,nrefw)
      case (2)      ! h k l       Fo^2     std        (file di tipo shelX)
        write(fileh%handle(),'(3i4,2x,2f8.2)')(refl(i)%hkl,refl(i)%fo**2,std,i=1,nrefw)
      case (3)      ! h k l       Fo^2     sigma      (file di tipo shelX)
        write(fileh%handle(),'(3i4,2x,2f8.2)')(refl(i)%hkl,0.01*refl(i)%fc**2,std,i=1,nrefw)
      case (4)      ! h k l       Fo       Fc     phase    (file fc)
        write(fileh%handle(),*)'  h   k  l      Fo      Fc     phase'
        write(fileh%handle(),'(3i4,3f8.2)')(refl(i)%hkl,refl(i)%fo,refl(i)%fc,real(refl(i)%ph),i=1,nrefw)
      case (5)
#if 0
        allocate(inte(nrefw),reflc(nrefw))
        inte(:) = refl(:nrefw)%m*refl(:nrefw)%lp*refl(:nrefw)%fc**2
        reflc(:) = refl(:nrefw)
        maxinte = maxval(inte)
        if (abs(maxinte) < epsilon(1.0)) then ! check for maxinte = 0
            maxinte = 1.0 
        else
            call sort_reflections(reflc,ORD_BY_INT,inte)   
            !call sort_reflections(reflc,ORD_BY_2THETA,inte)   
        endif
        nrefw = count(inte >= epsilon(1.0)*maxinte/10)  ! print only reflections with I% > 0
        write(fileh%handle(),'(a)')'  h   k  l      2theta    d      mult           LP'//    &
        '               Fc^2             Intensity                   Intensity(%)'
        write(fileh%handle(),'(3i4,2f10.4,i4,4f20.4)')                       & 
         (reflc(i)%hkl,reflc(i)%tthd,dvalue(reflc(i)%tthd),reflc(i)%m,reflc(i)%lp,reflc(i)%fc**2,   &
        ! (reflc(i)%hkl,reflc(i)%tthd,dvalue(reflc(i)%tthd),reflc(i)%m,reflc(i)%lp,reflc(i)%fc,   &
         inte(i),1000*(inte(i)/maxinte),i=1,nrefw)
#endif
      case (6)      ! h k l           Fc     phase 
        write(fileh%handle(),*)'  h   k  l      Fc     phase'
        write(fileh%handle(),'(3i4,2f8.2)')(refl(i)%hkl,refl(i)%fc,real(refl(i)%ph),i=1,nrefw)

   end select
!
   if (present(fname)) call fileh%fclose()
!
   end subroutine write_reflections

!----------------------------------------------------------------------------------------------------

   subroutine statvet(svety,svetx,nbin)
!   
!  Esegue statistica sul vettore svety ordinato   
!
   real, dimension(:), intent(in) :: svety ! vettore ordinato y
   real, dimension(:), intent(in) :: svetx ! vettore ordinato x della stessa dim. di y
   integer, intent(in)            :: nbin  ! numero di intervalli
   integer, dimension(nbin)       :: ivet
   integer                        :: iresto
   integer                        :: nsv
   integer                        :: i,nini,nfin
!   
!  calcola numero di valori per intervallo in ivet
   nsv = size(svety)
   ivet(:) = nsv / nbin
!   
!  gestisci il resto
   iresto = mod(nsv,nbin)
   ivet(:iresto) = ivet(:iresto) + 1
!   
!  stampa la statistica
   nini = 1
   nfin = 0
   write(6,'(/a)')'        n.       interval            value             R'
   do i=1,nbin
      nfin = nini + ivet(i) - 1
      write(6,'(i10,")",2x,i10,"-",i0,t30,f10.3,f10.3,f10.3)')i,nini,nfin,svetx(nini),svetx(nfin),sum(svety(nini:nfin))
      nini = nfin + 1
   enddo
!
   end subroutine statvet

!----------------------------------------------------------------------------------------------------

   subroutine write_reflection_table(table,vcol,kpri,ftype)
   USE strutil
   type(reflection_type), intent(in), dimension(:) :: table  ! tabella dei riflessi
   integer, dimension(:), intent(in)               :: vcol   ! quali colonne stampare
   integer, intent(in)                             :: kpri
   integer, intent(in)                             :: ftype  ! 1 -> F; 2 -> F^2
   integer, dimension(size(vcol))                  :: ktype
   integer, parameter                              :: NCOL = 8
   integer, dimension(NCOL)                        :: ival
   real, dimension(NCOL)                           :: rval
   character(len=20), dimension(7)                 :: sval
   integer                                         :: i
!
!  Titoli delle colonne
   ktype(:) = 3
   if (ftype == 1) then
       call set_row_list(vcol,ktype,sval=(/'n.     ','h k l  ','2-theta',    &
         'd      ','Fo     ','Fc     ','phase  ','m      '/),kpr=kpri)
   else
       call set_row_list(vcol,ktype,sval=(/'n.     ','h k l  ','2-theta',    &
         'd      ','Fo^2   ','Fc^2   ','phase  ','m      '/),kpr=kpri)
   endif
!
!  Scrivi la tabella dei riflessi
   ktype = (/1,3,23,23,23,23,1,1/)   ! formato delle colonne
   do i=1, size(table)
      ival(1) = i
      write(sval(2),'(3(i0,1x))')table(i)%hkl(:)
      rval(3) = table(i)%tthd
      rval(4) = table(i)%slaq
      rval(5) = table(i)%fo
      rval(6) = table(i)%fc
      ival(7) = table(i)%ph
      ival(8) = table(i)%m
      call set_row_list(vcol,ktype,ival,rval,sval,kpri)
   enddo
!
   end subroutine write_reflection_table

!-----------------------------------------------------------------------

   function LPcorrection(tthd)  result(lp)
!  Correzione Lorentz-polarizzazione
!      Formula usata :
!                     Lp = (1+kCos(2theta)**2) / (2*Sen(theta)**2*Cos(theta))
!                        = 0.5*(1+kCos(2theta)**2) * Csc(theta)**2*Sec(theta)
   USE General, only:neutro,nsyn,dtor
   real, dimension(:), intent(in) :: tthd
   real, dimension(size(tthd))    :: lp
   integer                        :: numb
   integer                        :: i
   real                           :: tth,th
   real                           :: Csc_th, Sec_th, Tan_th, Sin_th
   real                           :: kpol
!
   numb = size(tthd)
   if (neutro == 1 .or. nsyn == 1) then
       kpol = 0.0
   else
       kpol = 0.8
   endif
   do i=1,numb
      tth = tthd(i)*dtor
      th = 0.5 * tth
      Sin_th = Sin(th); Csc_th = 1.0/Sin_th ; Sec_th = 1.0/cos(th); Tan_th = Sin_th / Cos(th)
      lp(i) = 0.5*(1 + kpol*Cos(tth)**2)*Csc_th**2*Sec_th
   enddo
!
   end function LPcorrection

!-----------------------------------------------------------------------

   subroutine sort_reflections(refl,key,inte)
   USE nr
   type(reflection_type), dimension(:), intent(inout) :: refl
   integer, intent(in)                                :: key
   integer, dimension(size(refl))                     :: iord
   real, dimension(:), intent(inout), optional        :: inte
!
   select case(key)
          case (ORD_BY_2THETA)              ! ordina per 2theta
            call indexx(refl%tthd,iord)
            refl(:) = refl(iord)
            if (present(inte)) then
                inte(:) = inte(iord)
            endif

          case (ORD_BY_INT)                 ! ordina per intensita'
            call indexx(inte,iord)
            iord(:) = iord(size(inte):1:-1)
            refl(:) = refl(iord)
            inte(:) = inte(iord)
  
   end select
!
   end subroutine sort_reflections

!-----------------------------------------------------------------------

   subroutine save_refl_bin(unitbin,refl)
!
!  write reflections on binary file
!
   integer, intent(in)                                          :: unitbin
   type(reflection_type), dimension(:), allocatable, intent(in) :: refl
   integer                                                      :: nref
   nref = numrefl(refl)
   write(unitbin)nref
   if (nref > 0) then
       write(unitbin)refl(:)
   endif
   end subroutine save_refl_bin

!-----------------------------------------------------------------------

   subroutine read_refl_bin(unitbin,refl,err)
!
!  read reflections from binary file
!
   USE realloc
   USE errormod
   integer, intent(in)                                             :: unitbin
   type(reflection_type), dimension(:), allocatable, intent(inout) :: refl
   type(error_type), intent(out)                                   :: err
   integer                                                         :: ier
   integer                                                         :: nref
   read(unitbin, iostat=ier, err=10) nref
   if (nref > 0) then
       call reallocate(refl,nref,.false.)
       read(unitbin, iostat=ier, err=10)refl(:)
   endif
10 continue
   if (ier /= 0) then
       call err%set('Error on reading reflections')
   endif
   end subroutine

!-----------------------------------------------------------------------

   subroutine select_ref(refl,code)
!
!  Seleziona i reflessi in base ad un criterio definito da code  
!  
   USE realloc
   type(reflection_type), dimension(:), allocatable, intent(inout) :: refl
   integer, intent(in)                                             :: code
   integer, dimension(:), allocatable                              :: overl
   integer                                                         :: nrrf,nrrfnew
   integer                                                         :: i
   integer                                                         :: ovr
   
   select case (code)
      case (3)
!  
!       Seleziona riflessi singoli e doppietti
        nrrf = size(refl)
   
        allocate(overl(nrrf))
        call defovlexr(refl,0.6,overl)
        i = 0
        nrrfnew = 0
        do
           i = i + 1
           if (i > nrrf) exit
           ovr = overl(i)
           if (ovr == 1 .or. ovr == 2) then
               refl(nrrfnew+1:nrrfnew+ovr) = refl(i:i+1)
               nrrfnew = nrrfnew + ovr
               i = i + ovr - 1
           endif
        enddo
   
   end select
   call reallocate(refl,nrrfnew)
!   
   end subroutine select_ref

! -----------------------------------------------------------------

   subroutine create_reflections(rhomin,rhomax,cell,spg,kmat,tmat,nsym,jsys,icent,latt,refob,nrefl,ihmx)
   USE unit_cell
   USE realloc
   implicit none
   real, intent(in)                      :: rhomin
   real, intent(inout)                   :: rhomax
   character(len=*), intent(in)          :: spg   ! H-M space group symbol
   real, dimension(6), intent(in)        :: cell  ! cell parameters
   integer, dimension(:,:,:), intent(in) :: kmat  ! symmetry operators (rotation part)
   real, dimension(:,:), intent(in)      :: tmat  ! symmetry operators (translation part)
   integer, intent(in)                   :: nsym  ! number of symmetry operators (inversion and centering excluded)
   integer, intent(in)                   :: jsys  ! crystal system
   integer, intent(in)                   :: icent ! =1 if centric with -1 at origin
   integer, intent(in)                   :: latt 
   type(reflection_type), allocatable, dimension(:), intent(inout) :: refob 
   integer, intent(out)                  :: nrefl
   integer, dimension(3), intent(out)    :: ihmx
   real, dimension(6)                    :: pp
   real, dimension(3,3)                  :: matr
   integer :: klatt
   integer :: ksym
   integer :: iahmax, iakmax, ialmax
   integer :: ihmaxsav, ikmaxsav, ilmaxsav
   integer :: hmin,hmax,kmin,kmax,lmin,lmax
   integer, dimension(96) :: vsym
   integer, dimension(3) :: kv,i1
   integer, parameter :: NINITREF = 3000
   integer, parameter :: NMAXREF = 50000
   integer :: sizerefob
   integer :: i,j
   integer :: nr, jex, idelt, jcode, isgnn, id1
   integer :: nhmax, nkmax, nlmax
   integer :: indh, indk, indl, indh1, indk1, indl1
!corr   real :: rho, rhof, rho2, eps, twot, ctheta, rhomax1 - diff from expo because ctheta is in this module
   real :: rho, rho2, eps, twot, rhomax1
   integer :: multi, inbl
   integer :: ifl
   integer, dimension(6,13), parameter :: ibrav = reshape((/    &
                                                                1, 1, 1, 1, 3, 1,      &
                                                                0, 1, 1, 1, 0,-1,      &
                                                                1, 0, 1, 1, 1, 1,      &
                                                                1, 1, 0, 1, 1, 1,      &
                                                                2, 2, 2, 2, 2, 3,      &
                                                                0, 0, 0, 0, 1, 0,      &
                                                                0, 0, 0, 0, 0, 0,      &
                                                                0, 0, 0, 0, 1, 0,      &
                                                                0, 0, 0, 0, 2, 0,      &
                                                                0, 0, 0, 0, 1, 0,      &
                                                                0, 0, 0, 0, 1, 0,      &
                                                                0, 0, 0, 0, 0, 0,      &
                                                                0, 0, 0, 0, 2, 0       &
                                                               /),(/6,13/))

   iahmax = 0
   iakmax = 0
   ialmax = 0
   call reallocate(refob,NINITREF)
   sizerefob = NINITREF
!
   matr = matrice_mreciproca(cell)
   pp(:) = (/0.25*matr(1,1),0.25*matr(2,2),0.25*matr(3,3),0.5*matr(1,2),0.5*matr(1,3),0.5*matr(2,3)/)
!
!  ksym=number of translational matrices different from ( 0.0 0.0 0.0 )
   ksym=0
   do i=2,nsym
      ifl=0
      do j=1,3
         if (abs(tmat(i,j)).gt.0.00001) ifl=1
      enddo
      if (ifl.eq.0) cycle
      ksym=ksym+1
      vsym(ksym)=i
   enddo
   klatt = latt - 1
!
   nr=0
   ihmx(:) = 100
   kv(2)=2*ihmx(3)+1
   kv(3)=ihmx(2)*kv(2)+ihmx(3)
   kv(1)=kv(2)*(2*ihmx(2)+1)
!--
   call indmm(hmin,hmax,kmin,kmax,lmin,lmax,jsys,ihmx,spg)
!--
   nhmax=hmin
   nkmax=kmin
   nlmax=lmin
   rhomax1=rhomax
   ihmx(:)=-100
!--
   loop_indh: do indh=hmin,hmax
      do indk=kmin,kmax
         do indl=lmin,lmax
!---
!---        vedere se il nuovo riflesso generato e' equivalente
!---        ad uno gia' considerato, in questo caso :
!---
         if (indh.eq.0.and.indk.eq.0.and.indl.eq.0) cycle
!
!       TRIGONALE : va' considerata la condizione aggiuntiva |h|>|k|
!             nel caso (+-+)
         if (jsys.eq.5.and.indk.lt.0.and.iabs(indh).lt.iabs(indk)) cycle
!
!       CUBICO :  va' considerata la condizione aggiuntiva h<=k<=l
         if(jsys.eq.6.and.(indk.lt.indh.and.indk.gt.indl)) cycle
!
         indh1=indh
         indk1=indk
         indl1=indl
         rho=rhof(pp,indh1,indk1,indl1)
         rho2=rho*rho
         if(rho2.gt.rhomax.or.rho2.lt.rhomin) cycle
         call extin(jex,indh1,indk1,indl1,ibrav,klatt,ksym,vsym,nsym,kmat,tmat,icent)
         if (jex.eq.1) cycle
!
         ihmaxsav = iahmax
         ikmaxsav = iakmax
         ilmaxsav = ialmax
         call geneq(1,indh1,indk1,indl1,idelt,jcode,isgnn,id1,rho,kmat,tmat,icent,nsym,rhomax,ihmx,pp,iahmax,iakmax,ialmax)
         inbl=indh1*kv(1) + indk1*kv(2) + indl1 + kv(3)
         inbl=iabs(inbl)
         if(nr.gt.0)then
            if (any(refob(:nr)%ph == inbl)) cycle
         endif
         rhomax=rhomax1
         if (rho.gt.rhomax1.or.rho.lt.rhomin) then
             iahmax = ihmaxsav
             iakmax = ikmaxsav
             ialmax = ilmaxsav
             cycle
         endif
!
         if(iahmax.ge.nhmax) nhmax=iahmax
         if(iakmax.ge.nkmax) nkmax=iakmax
         if(ialmax.ge.nlmax) nlmax=ialmax
!
         nr=nr+1
         refob(nr)%ph=inbl  ! temporary save superindice in %ph
         if (nr == sizerefob) then
             call reallocate(refob,sizerefob+NINITREF,.true.)
             sizerefob = sizerefob + NINITREF
         endif
!
         i1(:) = [indh1,indk1,indl1]
!
!        Recompute rho2: necessary if cell is incompatible with s.g. (es. P 1 2 1 and angle alpha 90 90)
         rho2=pp(1)*i1(1)*i1(1) + pp(2)*i1(2)*i1(2) + pp(3)*i1(3)*i1(3) + pp(4)*i1(1)*i1(2) + pp(5)*i1(1)*i1(3) + pp(6)*i1(2)*i1(3)
!
         twot = ctheta(rho2)
         call cmult2002(i1,eps,multi)
         refob(nr)%hkl(:) = i1(:) 
         refob(nr)%phv = jcode
         refob(nr)%tthd = twot
         refob(nr)%m = multi
         if (nr == NMAXREF) exit loop_indh
         enddo
      enddo
   enddo loop_indh
   nrefl=nr
!
   ihmx(1)=nhmax
   ihmx(2)=nkmax
   ihmx(3)=nlmax
!
   call reallocate(refob,nrefl,.true.)
   if (nrefl == 0) return
!
!  sort reflections
   call sort_reflections(refob(:nrefl),ORD_BY_2THETA)
!
   end subroutine create_reflections

!------------------------------------------------------------
      subroutine indmm(hmin,hmax,kmin,kmax,lmin,lmax,jsys,ihmx,ntc)
      !USE General
      !USE Symmetry
      !USE sinterface
      implicit none
      integer  hmin,hmax,kmin,kmax,lmin,lmax                  
!corr      character ntc*16
      character(len=*), intent(in) :: ntc
      integer, intent(in) :: jsys
      integer, dimension(3), intent(in) :: ihmx
!corr      write(ntc,'(16a1)') nSpaceG
!corr     ntc = spaceg_string()
!corr          write(0,*)'NTC=',trim(ntc)
! --
! --   calcolo hmin e hmax , kmin e kmax , lmin e lmax                  
! --
        hmin=-ihmx(1)
        hmax=ihmx(1)
        kmin=-ihmx(2)
        kmax=ihmx(2)
        lmin=-ihmx(3)
        lmax=ihmx(3)
! --
      select case(jsys)
      case (1)    ! TRICLINO
        hmin=0
! --
! -- MONOCLINO
! --
      case (2)    ! MONOCLINO
       if (ntc(3:3).eq.'1') then
        if (ntc(5:5).eq.'1') then
! -- 2||c
            hmin=0
            lmin=0         
        else 
! -- 2||b
            hmin=0
            kmin=0         
        endif      
       else
        if (ntc(4:4).eq.'1') then
         if (ntc(6:6).eq.'1') then
! -- 2||a
            hmin=0
            kmin=0
         else
! -- 2||b
            hmin=0
            kmin=0         
         endif
        else
         if(ntc(5:5).eq.'1') then
! -- 2||a
            kmin=0
            hmin=0
         else
! -- 2||b
            hmin=0
            kmin=0         
         endif
        endif
       endif
! --
      case (3)   ! ORTOROMBICO
          hmin=0
          kmin=0
          lmin=0

      case (4)   ! TETRAGONALE
          hmin=0
          kmin=0
          lmin=0

      case (5)   ! ESAGONALE E TRIGONALE : 
          hmin=0
          lmin=0

      case (6)   ! CUBICO :  va' considerata la condizione aggiuntiva h<=k<=l
          hmin=0
          kmin=0
          lmin=0

      case (7)   ! RHOMBOHEDRIC CASE
          hmin=0
      end select
      return
      end subroutine indmm
!-----------------------------------------------------------------

      subroutine extin(jex,kg1,kg2,kg3,ibrav,klatt,ksym,vsym,nsym,kmat,tmat,icent)
      !USE General
      !USE Symmetry, only: tmat, kmat, icent, nsym
!   check for extinction (jex=0/1 allowed/absent)
!   first: check bravais lattice type
!   type  p   a  b    c   i    f   r
!   latt: 1   2   3   4   5    6   7
!   ibrav(    1   2   3   4    5   6  )  x  13
      implicit none
      integer :: jex, kg1, kg2, kg3, klatt, ksym
      integer ibrav(6,13),kg(3) !!!,vsym(48)    !!!!,mr(3)
      integer, intent(in)                   :: nsym
      integer, dimension(:,:,:), intent(in) :: kmat
      real, dimension(:,:), intent(in)      :: tmat
      integer, intent(in)                   :: icent
      integer, dimension(:) :: vsym
      integer, dimension(3) :: nhh
      integer :: nbrav, lsym
      integer :: i,j,jj
      integer :: ica, jc, kc, isum, lb, nh, isym
      real    :: ca
!
      kg(1)=kg1
      kg(2)=kg2
      kg(3)=kg3
      do 10 i=1,3
      if (kg(i).ne.0) go to 20
   10 continue
      jex=1
      return
   20 jex=0
      if(klatt.eq.0) go to 420
      nbrav=ibrav(klatt,1)
      do 410 i=1,nbrav
      isum=0
      do j=1,3
         lb=1+(i-1)*4+j
         isum=isum+kg(j)*ibrav(klatt,lb)
      enddo
      lb=4*i+1
      if(mod(isum,ibrav(klatt,lb)).eq.0) go to 410
      jex=1
      go to 700
410   continue
420   continue
!   second: check translations from symmetry operators
!         first condition for h to absent:  h*ri=h   (h=kg)
      if (ksym.eq.0) go to 700
      jc=icent+1
      do 490 kc=1,jc
      do 480 lsym=1,ksym
      isym=vsym(lsym)
      ca=0.
          if (kc == 2) then
              nhh(:) = matmul(kg,kmat(isym+nsym,:,:))
          else
              nhh(:) = matmul(kg,kmat(isym,:,:))
          endif
      do jj=1,3
!corr         nh=0
!corr         do j=1,3
!corr           mr(j)=0
!corr         enddo
!corr         do i=1,2
!corr            do j=1,3
!corr               ikk=ismat(i,j,isym)
!corr                ka=iabs(ikk)
!corr               if (ka == jj) then
!corr                   if (kc.eq.2) ikk=-ikk
!corr                   mr(j)=isign(1,ikk)
!corr               endif
!corr            enddo
!corr         enddo
!corr         do j=1,3
!corr            nh=kg(j)*mr(j)+nh
!corr         enddo
!corr         write(71,*)abs(nhh(jj)-nh),jj,'nh=',nh,nhh(jj),kg(jj)
         nh = nhh(jj)
         if(kg(jj).ne.nh)goto 480
!corr         ca=float(nh)*tsmat(jj,isym)   +  ca
         ca=float(nh)*tmat(isym,jj)   +  ca
      enddo
!  second condition:    h*ts ; integer, for an ext. reflection.
      ca=abs(ca)
      ica=(ca+0.0005)*1000
      if(mod(ica,1000).eq.0)goto 480
!  variable ca is not an integer number,the reflection is absent.
      jex=1
      goto 700
480     continue
490   continue
700     continue
      return
      end subroutine extin
!-----------------------------------------------------------------------
      subroutine geneq(lty,ih,kk,il,idelt,jcode,isnn,istd,rho,kmat,tmat,icent,nsym,rhomax,ihmx,pp,iahmax,iakmax,ialmax)
      implicit none
      integer, intent(in)                   :: lty
      integer, intent(inout)                :: ih,kk,il
      integer, intent(out)                  :: jcode, isnn
      integer, dimension(:,:,:), intent(in) :: kmat
      real, dimension(:,:), intent(in)      :: tmat
      integer, intent(in)                   :: icent
      integer, intent(in)                   :: nsym
      real, intent(inout)                   :: rhomax
      integer, dimension(3), intent(inout)  :: ihmx
      real, dimension(6), intent(in)        :: pp
      integer, intent(inout)                :: iahmax,iakmax,ialmax
      integer, dimension(nsym)              :: kl1,kl2,kl3
      integer, dimension(3)                 :: i1,i2,i1s
      integer                               :: i,j,ind,ido,maxi,ifl,istd,idelt
      integer                               :: jcode1,jcode2,jm1,ii1,ii2
      real                                  :: rho   !!!!!,rhof
      integer                               :: iah,iak,ial
!      common/indexmax/iah,iak,ial,iahmax,iakmax,ialmax
!      integer :: iah,iak,ial,iahmax,iakmax,ialmax
!
      isnn=1
      i1(1)=ih
      i1(2)=kk
      i1(3)=il
      jcode=13
      ido=1
      maxi=1
      do 250 j=1,nsym
         kl1(j)=2400
         kl2(j)=2400
         do 200 i=1,3
            i2(i)=0
            kl1(j)=kl1(j)-i1(i) * int(tmat(j,i)*24+0.01)
            kl2(j)=kl2(j)-i1(i) * int(tmat(j,i)*12+0.01)
  200    continue
         kl1(j)=mod(kl1(j),24)
         i2(:) = matmul(i1,kmat(j,:,:))
!
         if(icent.eq.1.or.ido.ne.1) goto 230
         jcode=1
         do 225 i=1,3
            if(i1(i)+i2(i).ne.0) goto 230
  225    continue
         jcode=mod(kl2(j),12)+1
         if(jcode.le.1) jcode=jcode+12
         ido=0
  230    ind=262144 * i2(1) + 512 * i2(2) + i2(3)
         kl3(j)=131328 + iabs(ind)
         rho=rhof(pp,i2(1),i2(2),i2(3))
         if (lty.eq.1) then
             rhomax=amax1(rhomax,rho)
             if(ihmx(1).lt.iabs(i2(1))) ihmx(1)=iabs(i2(1))
             if(ihmx(2).lt.iabs(i2(2))) ihmx(2)=iabs(i2(2))
             if(ihmx(3).lt.iabs(i2(3))) ihmx(3)=iabs(i2(3))
             rho=rho*rho
!
             ii1 = iabs(i1(1))
             ii2 = iabs(i2(1))
             iah=max0(ii1,ii2)
             if (iahmax.le.iah) iahmax = iah
!
             ii1 = iabs(i1(2))
             ii2 = iabs(i2(2))
             iak=max0(ii1,ii2)
             if (iakmax.le.iak) iakmax = iak
!
             ii1 = iabs(i1(3))
             ii2 = iabs(i2(3))
             ial=max0(ii1,ii2)
             if (ialmax.le.ial) ialmax = ial
!
         endif
         if(j.eq.1) goto 250
         jm1=j-1
         do 240 i=1,jm1
            if(kl3(i).eq.kl3(j)) kl3(j)=0
  240    continue
         if(kl3(j).le.kl3(maxi)) go to 250
         maxi=j
         isnn=isign(1,ind)
  250 continue
      call unpack (kl3(maxi),ih,kk,il)
      idelt=kl1(maxi)*15
      istd=0
      if (icent.eq.0) then
          ido=1
          i1s(1)=ih
          i1s(2)=kk
          i1s(3)=il
          do 300 j=1,nsym
             kl1(j)=24000
             do 260 i=1,3
             i2(i)=0
             kl1(j)=kl1(j)+i1s(i) * nint(tmat(j,i)*12)
  260        continue
             i2(:) = matmul(i1s,kmat(j,:,:))
             if (ido.eq.1) then
                 jcode=1
                 ifl = 0
                 do 290 i=1,3
                    if (i1s(i)+i2(i).ne.0) ifl = 1
  290            continue
                 if (ifl.eq.0) then
                     jcode1 = mod ( kl1(j) , 24 )
                     jcode2 = mod ( ( jcode1 + 12 ), 24 )
                     jcode = min0 ( jcode1 , jcode2 )
                     jcode = jcode + 1
                     if (jcode.le.1) jcode=jcode+12
                     ido=0
                   endif
               endif
  300     continue
!
          ii1 = iabs(i1s(1))
          ii2 = iabs(i2(1))
          iah=max0(ii1,ii2)
          if (iahmax.le.iah) iahmax = iah
          ii1 = iabs(i1s(2))
          ii2 = iabs(i2(2))
          iak=max0(ii1,ii2)
          if (iakmax.le.iak) iakmax = iak
          ii1 = iabs(i1s(3))
          ii2 = iabs(i2(3))
          ial=max0(ii1,ii2)
          if (ialmax.le.ial) ialmax = ial
!
      endif
!
      if (lty.eq.1) return
      if (i1(1).eq.ih.and.i1(2).eq.kk.and.i1(3).eq.il) istd=1
      return
      end subroutine geneq
!-----------------------------------------------------------------------
      subroutine unpack (ind,ih,kk,il)
      implicit none
      integer, intent(in)  :: ind
      integer, intent(out) :: ih, kk, il
      integer              :: is
      ih = ind/262144
      is = ind - 262144*ih
      kk = is/512-256
      il = is-512*(kk+256)-256
      return
      end subroutine unpack
! ----------------------------------------------------------------------
      real function rhof(p,jh,jk,jl)
      implicit none
      real, dimension(6), intent(in) :: p
      integer, intent(in)            :: jh,jk,jl
      real                           :: rho2
      rho2=p(1)*float(jh*jh)+p(2)*float(jk*jk)   &
       +p(3)*float(jl*jl)+p(4)*float(jh*jk)      &
       +p(5)*float(jh*jl)+p(6)*float(jk*jl)
      rhof=sqrt(rho2)
      end function rhof
!-----------------------------------------------------------------------
      real function ctheta(rho)
!
!-- compute 2-Theta value from ( sin(theta)/lambda ) ** 2
      USE General
      implicit none
      real, intent(in) :: rho
      real :: theta
      real :: alambda = 1.54056 !!! da sistemare
      theta=sqrt(rho)*alambda
      if (theta.gt.1.) theta=1.
      theta=asin(theta)*rtod
      ctheta=2.0*theta
      end function ctheta

END MODULE reflection_type_util   
